/* messages.c - Various messages returned to user.
 * 
 * Copyright (C) 2010, Pawel Krawczyk <pawel.krawczyk@hush.com> and
 * Jeroen Nijhof <jeroen@jeroennijhof.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program - see the file COPYING.
 *
 * See `CHANGES' file for revision history.
 */

char *protocol_err_msg = "(Protocol error)";
char *author_ok_msg = "(Service granted)";
char *author_fail_msg = "(Service not allowed)";
char *author_err_msg = "(Service not allowed. Server error)";
char *author_syserr_msg = "(Authorization system error)";
char *acct_ok_msg = "(Accounted ok)";
char *acct_fail_msg = "(Accounting failed)";
char *acct_err_msg = "(Accounting failed. Server error)";
char *acct_syserr_msg = "(Accounting system error)";
